package cks.model;

public class Location {

	public String fileName;
	public int line = -1;
	public int column = -1;

	private static final char FILE_LINE_SEP = ':';
	private static final char LINE_COL_SEP = '.';

	public Location(String fileName, int line, int column)
	{
		assert line > 0 : "bad line: " + line;
		assert column > 0 : "bad column: " + column;
		this.fileName = fileName;
		this.line = line;
		this.column = column;
	}

	public void toString(StringBuilder builder)
	{
		toStringFileName(builder);
		builder.append(FILE_LINE_SEP);
		builder.append(this.line);
		builder.append(LINE_COL_SEP);
		builder.append(this.column);
	}

	private void toStringFileName(StringBuilder builder)
	{
		if (this.fileName == null) {
			builder.append("<unknown-file>");
		} else {
			builder.append(this.fileName);
		}
	}

	public String toString()
	{
		StringBuilder builder = new StringBuilder();
		toString(builder);
		return builder.toString();
	}

	/**
	 * Shows this location relative to another location.	If the two
	 * locations are in different files, this function behaves the
	 * same as toString() does.	If both locations are in the same file,
	 * only the line and column are shown.	If both locations are in
	 * the same file and on the same line, only the column numbers
	 * are shown.	If both locations are exactly equal, the column number
	 * is still shown.
	 */
	public String toStringRelative(Location other)
	{
		StringBuilder builder = new StringBuilder();
		toStringRelative(builder, other);
		return builder.toString();
	}

	public void toStringRelative(StringBuilder builder, Location other)
	{
		if (this.fileName == null && other.fileName == null ||
				this.fileName.equals(other.fileName)) {

			if (this.line == other.line) {
				// "col <column>"
				builder.append("col ");
				builder.append(this.column);
			} else {
				// "<line>.<column>"
				builder.append(this.line);
				builder.append(LINE_COL_SEP);
				builder.append(this.column);
			}

		}
		else {
			// "<file>:<line>.<column>"
			toString(builder);
		}
	}

}
