package cks.model;

import java.util.Map;
import java.util.HashMap;

/**
 * A stack of mappings
 */
public class MapStack<K,V>
{
  private final Map<? super K,? extends V> map;
  private final MapStack<? super K,? extends V> next;

	public MapStack()
	{
		this(null);
	}

  public MapStack(Map<? super K,? extends V> map)
  {
    this(null, map);
  }

  public MapStack(MapStack<K,? extends V> next, Map<? super K,? extends V> map)
  {
    this.map = map;
    this.next = next;
  }

	public MapStack<K,V> push(Map<? super K,? extends V> contents)
  {
    return new MapStack<K,V>(this, contents);
  }

  public MapStack<? super K,? extends V> pop()
  {
    if (next == null) throw new RuntimeException("stack underflow");
    return next;
  }

  public V get(K ident)
  {
    if (ident == null) throw new IllegalArgumentException("null idents are not allowed");

		// Check our map
		if (map != null) {
			V value = map.get(ident);
			if (value != null) return value;
		}

		if (next == null) return null;
		return next.get(ident);
  }

  public void dump(java.io.PrintStream out)
  {
    out.append("------------------------------------\n");

    // Dump our own map
    if (map != null) {
      for (Map.Entry<? super K,? extends V> entry : map.entrySet()) {
        out.append(entry.getKey() + ": " + entry.getValue().getClass().getName() + "\n");
      }
    }

    // Dump out next map down the line
    if (this.next != null) {
      this.next.dump(out);
    }
  }

	// -----------------------------------------------------
	// empty instance

	private static final MapStack<Object,Object> EMPTY = new MapStack<Object,Object>(null);

	@SuppressWarnings("unchecked")
	public static final <K,V> MapStack<K,V> empty()
	{
		return (MapStack<K,V>) EMPTY;
	}

}
