package cks.model.type;

import cks.model.*;
import java.util.List;
import java.util.Arrays;
import java.util.Collections;

// def TKind = Base | Function TKind TKind
public abstract class TKind
{
  private TKind() {}

	public final TKind abs() { return new Function(BASE_LIST, this); }
	public final TKind abs(List<TKind> params)
	{
		return new Function(params, this);
	}

	public boolean equals(Object o)
	{
		assert o instanceof TKind : "o.class = " + o.getClass().getName();
		return equals((TKind) o);
	}

	public abstract boolean equals(TKind o);

	public String toString()
	{
		StringBuilder b = new StringBuilder();
		toString(b);
		return b.toString();
	}

	public abstract void toString(StringBuilder b);

	public static class Function extends TKind
	{
		public final List<TKind> params;
		public final TKind result;

		public Function(List<TKind> params, TKind result)
		{
			assert params != null;
			assert result != null;

			this.params = params;
			this.result = result;
		}

		public boolean equals(TKind o)
		{
			if (!(o instanceof Function)) return false;
			Function f = (Function) o;
			return this.params.equals(f.params) && this.result.equals(f.result);
		}

		public void toString(StringBuilder b)
		{
			b.append('(');
			if (params.size() > 0) {
				for (TKind param : params) {
					param.toString(b);
					b.append(", ");
				}
				b.setLength(b.length() - 1);
			}
			b.append(") -> ");
			result.toString(b);
		}
	}

	public static class Base extends TKind
	{
		private Base() {}
		public static final TKind INSTANCE = new Base();

		public boolean equals(TKind o)
		{
			return o == INSTANCE;
		}

		public String toString() { return "*"; }
		public void toString(StringBuilder b) { b.append(toString()); }
	}

	public static final TKind BASE = Base.INSTANCE;
	public static final List<TKind> BASE_LIST = Collections.singletonList(BASE);
}
