package cks.model.type;

import cks.model.*;

import java.io.PrintWriter;
import java.util.Map;
import java.util.TreeMap;
import java.util.List;

public class TRef extends TIndirect
{
	public final String ident;
	public TNamed target; // Filled in during resolution
	public TKind kind; // Filled in during resolution

	public TRef(Location loc, String ident)
	{
		super(loc);
		assert ident != null;
		this.ident = ident;
	}

	// resolve --------------------------------------------

	protected void resolveNames_(MapStack<String,TNamed> context)
		throws ResolutionException
	{
		TNamed match = context.get(this.ident);
		if (match == null) {
			throw new ResolutionException(this, "undefined type \"" + this.ident + "\"");
		}
		this.target = match;
	}

	protected void resolveKinds_() { }

	private boolean calculatingKind = false;

	public void calculateKind()
		throws ResolutionException
	{
		if (kind != null) return;

		if (calculatingKind) {
			throw new ResolutionException(this, "cyclic definition");
		}

		calculatingKind = true;
		target.calculateKind();
		kind = target.getKind();
		calculatingKind = false;
	}

	// properties --------------------------------------------

	public TKind getKind()
	{
		assert kind != null : this + " @ " + this.loc;
		return this.kind;
	}
	public TCompound getBase() {
		System.out.println("target: " + target.getClass().getName());
		return target.getBase();
	}
	public String getIdent() { return ident; }

	// dump --------------------------------------------

	public void toString(StringBuilder b)
	{
		super.toString(b);
		b.append(": ");
		b.append(this.ident);
	}
}
