package cks.model.type;

import cks.model.*;

import java.io.PrintWriter;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

public abstract class Type extends Node
{
	public static final List<Type> EMPTY_LIST = new ArrayList<Type>(0);
	protected boolean isResolved = false;
	private TKind kind;

	public Map<String,TDef> defs;

	public Type(Location loc)
	{
		super(loc);
	}

	public Type()
	{
		super();
	}

	protected void dumpChildren(PrintWriter out, int depth)
	{
		if (defs != null && defs.size() > 0) {
			indent(out, depth);
			out.println("[defs]");
			for (TDef def : defs.values()) {
				def.dump(out, depth + 1);
			}
		}

		super.dumpChildren(out, depth);
	}

	public final boolean isResolved() { return this.isResolved; }

	public final void resolve(MapStack<String,TNamed> context)
		throws ResolutionException
	{
		assert context != null;
		assert !this.isResolved;
		resolveNames(context);
		resolveKinds();
		this.isResolved = true;
	}
	
	protected final void resolveNames(MapStack<String,TNamed> context)
		throws ResolutionException
	{
		assert context != null;

		if (this.defs != null) {
			context = context.push(this.defs);
			for (TDef def : this.defs.values()) {
				def.resolveNames(context);
			}
		}

		PrintWriter out = new PrintWriter(System.out);

		/*
		out.print("+ ");
		out.print(this);
		out.print(" | ");
		out.print(this.loc);
		out.println();
		out.flush();
		*/

		this.resolveNames_(context);

		/*
		out.print("- ");
		out.print(this);
		out.print(" | ");
		out.print(this.loc);
		out.println();
		out.flush();
		*/
	}

	protected final void resolveKinds()
		throws ResolutionException
	{
		calculateKind();

		if (this.defs != null) {
			for (TDef def : this.defs.values()) {
				def.resolveKinds();
			}
		}

		resolveKinds_();
	}

	public abstract TKind getKind();

	public abstract TCompound getBase();

	protected abstract void resolveNames_(MapStack<String,TNamed> context)
		throws ResolutionException;

	protected abstract void resolveKinds_()
		throws ResolutionException;

	protected abstract void calculateKind()
		throws ResolutionException;

	//public abstract void accept(TypeVisitor v);

	// Standard Types
	public static final TOpaque VOID		= new TOpaque("()");
	public static final TOpaque LIST		= new TOpaque("[]", TKind.BASE.abs());
	public static final TOpaque MAYBE	 = new TOpaque("?", TKind.BASE.abs());
	public static final TOpaque STRING	= new TOpaque("String");
	public static final TOpaque INTEGER = new TOpaque("Integer");
	public static final TOpaque BOOLEAN = new TOpaque("Boolean");

	public static final MapStack<String,TNamed> EMPTY_CONTEXT = MapStack.empty();

	// Type names that are available by default.
	public static final MapStack<String,TNamed> STANDARD_CONTEXT;

	static {
		HashMap<String,TOpaque> m = new HashMap<String,TOpaque>();
		m.put("String", STRING);
		m.put("Integer", INTEGER);
		m.put("Boolean", BOOLEAN);
		STANDARD_CONTEXT = new MapStack<String,TNamed>(m);
	}

	public static class Visitor
	{
		public void visit(Type t)				{ throw new RuntimeException("unhandled visitor case: " + t.getClass().getName()); }

		public void visit(TCompound t)	 { visit((Type) t); }
		public void visit(TRecord t)		 { visit((TCompound) t); }
		public void visit(TVariant t)		{ visit((TCompound) t); }

		public void visit(TIndirect t)	 { visit((Type) t); }
		public void visit(TRef t)				{ visit((TRef) t); }
		public void visit(TInvoke t)		 { visit((TInvoke) t); }
	}
}
