package cks.model.value;

import cks.model.*;
import java.math.BigInteger;

public abstract class VPrimitive extends Value
{
	public VPrimitive(Location loc)
	{
		super(loc);
	}

	public static class Void extends VPrimitive
	{
		public Void(Location loc)
		{
			super(loc);
		}
	}

	public static class String extends VPrimitive
	{
		public java.lang.String value;

		public String(Location loc, java.lang.String value)
		{
			super(loc);

			assert value != null;
			this.value = value;
		}

		public void toString(StringBuilder b)
		{
			super.toString(b);
			b.append(": ");
			b.append('\"');
			toJavaString(b, this.value);
			b.append('\"');
		}
	}

	public void toJavaString(StringBuilder b, java.lang.String s)
	{
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c >= 32 && c <= 126) {
				b.append(c); // normal ASCII character
			} else if (c == '\n') {
				b.append("\\n");
			} else if (c == '\r') {
				b.append("\\r");
			} else if (c == '\t') {
				b.append("\\t");
			} else if (c == '"') {
				b.append("\\\"");
			} else {
				b.append("\\u"); // weird forin character
				final java.lang.String HEX = "0123456789abcdef";
				int code = (char) c;
				b.append(HEX.charAt((code >> 12) & 0xf));
				b.append(HEX.charAt((code >>  8) & 0xf));
				b.append(HEX.charAt((code >>  4) & 0xf));
				b.append(HEX.charAt((code >>  0) & 0xf));
			}
		}
	}


	public static class Integer extends VPrimitive
	{
		public final BigInteger value;

		public Integer(Location loc, BigInteger value)
		{
			super(loc);
			this.value = value;
		}

		public void toString(StringBuilder b)
		{
			super.toString(b);
			b.append(": ");
			b.append(this.value);
		}
	}

	public static class Boolean extends VPrimitive
	{
		public final boolean value;

		public Boolean(Location loc, boolean value)
		{
			super(loc);
			this.value = value;
		}

		public void toString(StringBuilder b)
		{
			super.toString(b);
			b.append(": ");
			b.append(this.value);
		}
	}

	public static class Maybe extends VPrimitive
	{
		public final Value value;

		public Maybe(Location loc, Value value)
		{
			super(loc);
			this.value = value;
		}
	}
}
