package cks.parser;

import java.io.IOException;

public final class SimpleTokenManager
{
  private TokenSource source;
  private Token next; // lookahead
	private boolean prevWasEol;

  public SimpleTokenManager(TokenSource source)
  {
    this.source = source;
    this.next = null;
		this.prevWasEol = false;
  }

	private Token get()
		throws SyntaxException, IOException
	{
		this.prevWasEol = false;
		while (true) {
			Token token = source.get();

			if (token.type == Codes.Lead) {
				// skip
			} else if (token.type == Codes.EOL) {
				// skip, but remember that we saw it
				this.prevWasEol = true;
			} else {
				return token;
			}

		}
	}

  // take
  public Token take()
    throws SyntaxException, IOException
  {
    Token token;
    if (next == null) {
			token = get();
    } else {
      token = next;
      next = null;
    }

    return token;
  }

  // peek
  public Token peek()
    throws SyntaxException, IOException
  {
    if (next == null) {
			next = get();
    }
    return next;
  }

  // next
  public void next()
    throws SyntaxException, IOException
  {
    if (next == null) {
      throw new RuntimeException("you're not allowed to ignore tokens, punk");
    }
    next = null;
  }

	public boolean prevWasEol() { return this.prevWasEol; }

  // expect(token)
  public void expect(Token token, int type)
    throws SyntaxException
  {
    if (token.type != type) {
      StringBuilder error = new StringBuilder("expecting ");
      Token.toStringContent(error, type);
      throw new SyntaxException(token, error.toString());
    }
  }

  // expect
  public Token expect(int type)
    throws SyntaxException, IOException
  {
		Token t = take();
    expect(t, type);
		return t;
  }

}

