package cks.parser;

import cks.model.Location;
import java.math.BigInteger;

public class Token {

  public final int type;
  public String text;
  public Location loc;

  public static final Token[] EMPTY_ARRAY = new Token[0];

  public Token(int type)
  {
    this.type = type;
  }

  public static class LitInt extends Token
  {
    public final BigInteger value;
    public LitInt(BigInteger value)
    {
      super(Codes.LitInt);
      this.value = value;
    }
  }

  public static class LitStr extends Token
  {
    public final String value;
    public LitStr(String value)
    {
      super(Codes.LitString);
      this.value = value;
    }
  }

  public String toString()
  {
    StringBuilder builder = new StringBuilder();
    toString(builder);
    return builder.toString();
  }

  public void toString(StringBuilder builder)
  {
    toStringContent(builder);
    builder.append(" (");
    if (loc == null) {
      builder.append("[no location info]");
    } else {
      loc.toString(builder);
    }
    builder.append(")");
  }

  public String toStringContent()
  {
    StringBuilder builder = new StringBuilder();
    toStringContent(builder);
    return builder.toString();
  }

  public void toStringContent(StringBuilder builder)
  {
    String literal;
    String name;
    if (this.type < Codes.FIRST_CODE) {
      toStringLiteral(builder);
    } else {
      int index = this.type - Codes.FIRST_CODE;

      boolean displayLiteral = Codes.DISPLAY_LITERAL[index];
      boolean displayName = Codes.DISPLAY_NAME[index];
      if (displayLiteral) {
        toStringLiteral(builder);
        if (displayName) {
          builder.append(" [");
          builder.append(Codes.NAMES[index]);
          builder.append(']');
        }
      } else if (displayName) {
        builder.append('[');
        builder.append(Codes.NAMES[index]);
        builder.append(']');
      } else {
        throw new RuntimeException("nothing to display for token: " + this);
      }
    }
  }

  public static String toStringContent(int type)
  {
    StringBuilder builder = new StringBuilder();
    toStringContent(builder, type);
    return builder.toString();
  }

  public static void toStringContent(StringBuilder builder, int type)
  {
    if (type < Codes.FIRST_CODE) {
      toStringLiteral(builder, (char) type);
    } else {
      int index = type - Codes.FIRST_CODE;
      builder.append(Codes.NAMES[index]);
    }
  }

  public void toStringLiteral(StringBuilder builder)
  {
    if (this.text.length() == 1) {
      // if it's only one char
      char c = this.text.charAt(0);
      toStringLiteral(builder, c);
    } else {
      // if it's a bunch of characters, print it out quoted.  Use
      // '<' '>' if the text is already double quoted.
      char first = this.text.charAt(0);
      if (first == '"' || first == '\'' || first == '`') {
        builder.append('{');
        builder.append(this.text);
        builder.append('}');
      } else {
        builder.append('"');
        builder.append(this.text);
        builder.append('"');
      }
    }
  }

  public static void toStringLiteral(StringBuilder builder, char c)
  {
    if (c >= 20 && c <= 126) {
      // it's printable
      if (c == '\'' || c == '"' || c == '`') {
        // enclose in parens if it's a quote-ish character
        builder.append('(');
        builder.append(c);
        builder.append(')');
      } else { 
        // enclose in quotes otherwise
        builder.append('"');
        builder.append(c);
        builder.append('"');
      }
    } else {
      // it's not printable
      builder.append("[#");
      builder.append(Character.getNumericValue(c));
      builder.append("]");
    }
  }
  
}
